/*****************************************************************************
*
* Atmel Corporation
*
* File              : usi_logger_slave.h (usi_twi_slave.h)
* Compiler          : WINAVR
* Revision          : $Revision: 1.7 $
* Date              : $Date: 26. mai 2004 11:35:34 $
* Updated by        : $Author: ltwa $
*
* Support mail      : avr@atmel.com
*
* Supported devices : All device with USI module can be used.
*                     The example is written for the ATmega169, ATtiny26 & ATtiny2313
*
* AppNote           : AVR312 - Using the USI module as a TWI slave
*
* Description       : Header file for USI_TWI driver
*
* Changes for WinAVR by Frank Jonischkies, fjonisch@gmx.de, 11.08.2005
* fr den USI-Logger angepasste, vereinfachte Version
* Tabwidth			  : 3
*
****************************************************************************/

#define TWI_RX_BUFFER_SIZE 64		// muss geradzahlig sein, damit die berprfung
											// der Indexgrenzen funktioniert
											// ausserdem hngt die Interpretation der Daten davon ab,
											// ob sie an 'geradem' oder 'ungeradem' Index abgelegt sind
#define TWI_BUFFER_MASK (TWI_RX_BUFFER_SIZE - 1)
// wenn man TWI_RX_BUFFER_SIZE auf 2^n bringen kann, dann ist die Begrenzung des
// Index mittel der TWI_BUFFER_MASK eleganter und Programmspeicher sparend.
// Allerdings gibt es bei BUFFER_SIZE von 64 bereits einen Speicherberlauf.
// Wird ohne die LCD-Ausgabe und die erforderlichen Variablen gearbeitet, 
// dann kann BUFFER_SIZE auf 64 (oder auch hher ?) gesetzt werden.

#define ACK		','	// diese Zeichen werden ausgegeben jeweils fr:
#define NACK	';'
#define STOP	'>'
#define START	'<'

#define TRUE	1
#define FALSE	0

#define ATTINY2313

//********** USI_TWI Prototypes **********//

void     usi_twi_slave_init( void );
uint8_t 	usi_twi_get_byte( void );


// Device dependant defines
#if defined (__AVR_ATmega169__)
#define DDR_USI 		DDRE
#define PORT_USI		PORTE
#define PIN_USI 		PINE
#define PORT_USI_SDA	PORTE5
#define PORT_USI_SCL	PORTE4
#define PIN_USI_SDA 	PINE5
#define PIN_USI_SCL 	PINE4

#elif defined (__AVR_ATtiny26__)
#define DDR_USI 		DDRB
#define PORT_USI		PORTB
#define PIN_USI 		PINB
#define PORT_USI_SDA	PORTB0
#define PORT_USI_SCL	PORTB2
#define PIN_USI_SDA 	PINB0
#define PIN_USI_SCL 	PINB2

#elif defined (ATTINY2313)
#define DDR_USI 		DDRB
#define PORT_USI 		PORTB
#define PIN_USI 		PINB
#define PORT_USI_SDA	PB5
#define PORT_USI_SCL	PB7
#define PIN_USI_SDA 	PINB5
#define PIN_USI_SCL 	PINB7

#else
#warning "device type not defined"
#endif

